import bpy

class MspGLMeshProperties(bpy.types.Panel):
	bl_idname = "MESH_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "data"

	@classmethod
	def poll(cls, context):
		return context.active_object.type=="MESH"

	def draw(self, context):
		mesh = context.active_object.data

		self.layout.prop(mesh, "winding_test")
		self.layout.prop(mesh, "smoothing")

		self.layout.separator()

		col = self.layout.column()
		col.label("Data selection")
		col.prop(mesh, "use_lines")
		col.prop(mesh, "vertex_groups")
		col.prop(mesh, "max_groups_per_vertex")

		self.layout.separator()

		col = self.layout.column()
		col.label("Texturing")
		col.prop(mesh, "use_uv")
		col.prop(mesh, "tbn_vecs")
		col.prop(mesh, "tbn_uvtex")

class MspGLObjectProperties(bpy.types.Panel):
	bl_idname = "OBJECT_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "object"

	def draw(self, context):
		obj = context.active_object

		self.layout.prop(obj, "compound")
		self.layout.prop(obj, "lod_for_parent")
		if obj.lod_for_parent:
			self.layout.prop(obj, "lod_index")

class MspGLMaterialProperties(bpy.types.Panel):
	bl_idname = "MATERIAL_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "material"

	@classmethod
	def poll(cls, context):
		return context.active_object.active_material is not None

	def draw(self, context):
		mat = context.active_object.active_material
		if not mat:
			return

		self.layout.prop(mat, "technique")
		self.layout.prop(mat, "inherit_tech")
		if mat.inherit_tech:
			self.layout.prop(mat, "override_material")
		self.layout.prop(mat, "srgb_colors")
		self.layout.prop(mat, "array_atlas")
		if mat.array_atlas:
			self.layout.prop(mat, "array_layer")
		self.layout.prop(mat, "material_map")

class MspGLTextureProperties(bpy.types.Panel):
	bl_idname = "TEXTURE_PT_mspgl_properties"
	bl_label = "MspGL properties"
	bl_space_type = "PROPERTIES"
	bl_region_type = "WINDOW"
	bl_context = "texture"

	@classmethod
	def poll(cls, context):
		mat = context.active_object.active_material
		return mat is not None and mat.active_texture is not None

	def draw(self, context):
		tex = context.active_object.active_material.active_texture
		if not tex:
			return

		self.layout.prop(tex, "default_filter")

def register_properties():
	bpy.types.Mesh.winding_test = bpy.props.BoolProperty(name="Winding test", description="Perform winding test to skip back faces")
	bpy.types.Mesh.smoothing = bpy.props.EnumProperty(name="Smoothing", description="Smoothing method to use", default="MSPGL",
		items=(("NONE", "None", "No smoothing"),
			("BLENDER", "Blender", "Use Blender's vertex normals"),
			("MSPGL", "MspGL", "Compute vertex normals internally")))
	bpy.types.Mesh.use_lines = bpy.props.BoolProperty(name="Include lines", description="Include edges without faces as lines", default=False)
	bpy.types.Mesh.vertex_groups = bpy.props.BoolProperty(name="Vertex groups", description="Include vertex groups and weights", default=False)
	bpy.types.Mesh.max_groups_per_vertex = bpy.props.IntProperty(name="Max groups", description="Maximum amount of groups per vertex", min=1, max=4, default=2)
	bpy.types.Mesh.use_uv = bpy.props.EnumProperty(name="Use UV", description="Use UV coordinates", default="UNIT0",
		items=(("NONE", "None", "Ignore all UV coordinates"),
			("UNIT0", "Unit 0", "Use UV coordinates for unit 0"),
			("ALL", "All", "Use all UV coordinates")))
	bpy.types.Mesh.tbn_vecs = bpy.props.BoolProperty(name="TBN vectors", description="Compute tangent and binormal vectors for vertices", default=False)
	bpy.types.Mesh.tbn_uvtex = bpy.props.StringProperty(name="TBN UV layer", description="UV layer to use as basis for TBN vectors", default="")

	bpy.types.Object.compound = bpy.props.BoolProperty(name="Compound with parent", description="Join this object to its parent when exporting")
	bpy.types.Object.lod_for_parent = bpy.props.BoolProperty(name="LoD for parent", description="This object is a level of detail for its parent")
	bpy.types.Object.lod_index = bpy.props.IntProperty(name="LoD index", description="Index of the level of detail", min=1, max=16, default=1)

	bpy.types.Material.technique = bpy.props.StringProperty(name="Technique", description="Name of an external technique to use for rendering")
	bpy.types.Material.inherit_tech = bpy.props.BoolProperty(name="Inherit technique", description="Inherit from the technique to customize textures")
	bpy.types.Material.override_material = bpy.props.BoolProperty(name="Override material", description="Override material in the inherited technique as well", default=True)
	bpy.types.Material.srgb_colors = bpy.props.BoolProperty(name="sRGB colors", description="Export material colors as sRGB instead of linear", default=True)
	bpy.types.Material.array_atlas = bpy.props.BoolProperty(name="Texture array atlas", description="The material is stored in a texture array")
	bpy.types.Material.array_layer = bpy.props.IntProperty("Texture array layer", description="Layer of the texture array atlas to use")
	bpy.types.Material.material_map = bpy.props.BoolProperty(name="Material map", description="Make this material part of a material map")

	bpy.types.Texture.default_filter = bpy.props.BoolProperty(name="Default filter", description="Let the loading program determine filtering options")
